@@ -1,6 +1,6 @@
 /*  call.h -- variant calling declarations.
 
-    Copyright (C) 2013-2014 Genome Research Ltd.
+    Copyright (C) 2013-2015, 2019-2020 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -34,7 +34,7 @@ THE SOFTWARE.  */
 #define CALL_CONSTR_TRIO    (1<<2)
 #define CALL_CONSTR_ALLELES (1<<3)
 //
-//
+#define CALL_FMT_PV4        (1<<5)
 #define CALL_FMT_GQ         (1<<6)
 #define CALL_FMT_GP         (1<<7)
 
@@ -52,18 +52,13 @@ family_t;
 // For the single-sample and grouped -G calling
 typedef struct
 {
+    double ref_lk, max_lk, lk_sum;
     float *qsum;    // QS(quality sum) values
-    int nqsum, dp;
-    double fa,fb,fc,fa2,fb2,fc2,fab,fac,fbc;
+    int nqsum;
+    uint32_t *smpl, nsmpl;
+    uint32_t nals, als;
 }
-grp1_t;
-typedef struct
-{
-    grp1_t *grp;
-    int ngrp;
-    int *smpl2grp;
-}
-grp_t;
+smpl_grp_t;
 
 // For the `-C alleles -i` constrained calling
 typedef struct
@@ -82,6 +77,7 @@ typedef struct
     int *pl_map, npl_map;   // same as above for PLs, but reverse (new -> old)
     char **als;             // array to hold the trimmed set of alleles to appear on output
     int nals;               // size of the als array
+    int als_new, nals_new;  // bitmask with final alleles and their number
     family_t *fams;         // list of families and samples for trio calling
     int nfams, mfams;
     int ntrio[5][5];        // possible trio genotype combinations and their counts; first idx:
@@ -96,18 +92,16 @@ typedef struct
     int32_t *ugts, *cgts;   // unconstraind and constrained GTs
     uint32_t output_tags;
     char *prior_AN, *prior_AC;  // reference panel AF tags (AF=AC/AN)
-    tgt_als_t *tgt_als;     // for CALL_CONSTR_ALLELES
-    char *sample_groups;    // for single-sample or grouped calling with -G
-    grp_t smpl_grp;
-    float *qsum;
-    int nqsum;
+    tgt_als_t *tgt_als;         // for CALL_CONSTR_ALLELES
+    char *sample_groups;        // for single-sample or grouped calling with -G
+    char *sample_groups_tag;    // for -G [AD|QS:]
+    smpl_grp_t *smpl_grp;
+    int nsmpl_grp;
 
     // ccall only
     double indel_frac, min_perm_p, min_lrt;
     double prior_type, pref;
-    double ref_lk, lk_sum;
     int ngrp1_samples, n_perm;
-    int nhets, ndiploid;
     char *prior_file;
     ccall_t *cdat;
 
@@ -149,7 +143,7 @@ void qcall_destroy(call_t *call);
 void call_init_pl2p(call_t *call);
 uint32_t *call_trio_prep(int is_x, int is_son);
 
-void init_allele_trimming_maps(call_t *call, int als, int nals);
-void mcall_trim_numberR(call_t *call, bcf1_t *rec, int nals, int nout_als, int out_als);
+void init_allele_trimming_maps(call_t *call, int nals_ori, int als_out);
+void mcall_trim_and_update_numberR(call_t *call, bcf1_t *rec, int nals_ori, int nals_new);
 
 #endif
